class ExtendedLogger
  class ColorScheme
    def self.build values=nil
      values ||= {}

      instance = new
      Palette.apply values, instance
      instance
    end

    def colorize message
      value = get message.level.name
      message.prose = value.colorize message.prose
    end

    def get level_name
      table[level_name]
    end
    alias_method :[], :get

    def reset
      "\e[0m"
    end

    def set level_name, fg: nil, bg: nil
      value = get level_name
      value.fg = fg if fg
      value.bg = bg if bg
    end

    def table
      @table ||= Hash.new do |hash, level_name|
        hash[level_name] = Value.new
      end
    end

    None = self.new

    module Palette
      def self.apply values, color_scheme
        values.each do |level_name, values|
          values = { :fg => values } if values.is_a? Symbol

          values.each_key do |target|
            color_name = values[target]
            values[target] = Palette.get color_name
          end

          color_scheme.set level_name, **values
        end
      end

      def self.get name
        names.index name
      end

      def self.names
        %i(
          black
          red
          green
          brown
          blue
          magenta
          cyan
          gray
          dark_gray
          bright_red
          bright_green
          yellow
          bright_blue
          bright_magenta
          bright_cyan
          white
        )
      end
    end

    Value = Struct.new :fg, :bg do
      def colorize prose
        return prose if fg.nil? and bg.nil?

        new_prose = String.new
        new_prose << escape_code
        new_prose << prose
        new_prose << reset
        new_prose
      end

      def escape_code
        string = String.new "\e["

        if fg
          brightness, fg_color = fg.divmod 8
          string << brightness.to_s
          string << ';3'
          string << fg_color.to_s
        end

        if bg
          string << ';' if fg

          _, bg_color = bg.divmod 8
          string << '4'
          string << bg_color.to_s
        end

        string << 'm'

        string
      end

      def reset
        "\e[0m"
      end
    end
  end
end
