class ExtendedLogger
  def self.define levels=nil, **sink_defaults
    levels ||= Defaults.levels

    subclass = Class.new self
    Define.(subclass, levels, sink_defaults)
    subclass
  end

  class Define
    attr_reader :levels
    attr_reader :sink_defaults
    attr_reader :subclass

    def initialize subclass, levels, sink_defaults
      @levels = levels
      @sink_defaults = sink_defaults
      @subclass = subclass
    end

    def self.call *arguments
      instance = new *arguments
      instance.()
    end

    def call
      levels = self.level_set

      subclass.define_singleton_method :levels do levels end
      subclass.send :define_method, :levels do self.class.levels end

      if sink_defaults = self.sink_defaults
        subclass.define_singleton_method :sink_defaults do
          sink_defaults
        end
      end

      levels.each do |level|
        method_name = level.name

        subclass.send :define_method, method_name do |*arguments, &block|
          add level, *arguments, &block
        end
      end
    end

    def level_set
      if levels.is_a? Level::Set
        levels
      else
        Level::Set.build levels
      end
    end
  end
end
