class ExtendedLogger
  attr_writer :clock
  attr_accessor :label
  attr_writer :level
  attr_reader :sink

  def initialize sink
    @sink = sink
  end

  def self.build device=nil, level: nil, **sink_arguments
    sink = self.sink device: device, **sink_arguments

    instance = new sink
    instance.level = level if level
    instance
  end

  def self.empty_message
    '<empty message>'
  end

  def self.levels
  end

  def self.sink device: nil, **sink_arguments
    sink_arguments = sink_defaults.merge sink_arguments

    sink_arguments[:levels] ||= levels if levels

    Sink.build device, **sink_arguments
  end

  def self.sink_defaults
    {}
  end

  def << message
    sink << message
  end

  def add level, prose=nil, label: nil, &block
    block ||= ->{ prose }
    label ||= self.label

    time = clock.now

    message = Message.new block, level, time, label

    self << message
  end

  def clock
    @clock ||= Time
  end

  def device
    sink.device
  end

  def level
    sink.level
  end

  def level= reference
    sink.level = reference
  end

  def lower_verbosity
    sink.lower_verbosity
  end

  def raise_verbosity
    sink.raise_verbosity
  end

  module Assertions
    def nothing_written?
      written? ''
    end

    def written? expected_data
      device.rewind
      actual_data = device.read

      case expected_data
      when Regexp then expected_data.match actual_data
      else actual_data == expected_data.to_s
      end
    end
  end
end
