class ExtendedLogger
  class Level
    class Set
      attr_reader :default_name
      attr_reader :names

      def initialize names, default_name=nil
        @default_name = default_name
        @names = names
      end

      def self.build names, default: nil
        default ||= names.first
        new names, default
      end

      def == other
        default_name == other.default_name and
          names == other.names
      end

      def build_index
        hash = {}

        names.reduce nil do |previous_level, name|
          level = Level.new name

          level.prev = previous_level
          previous_level.next = level if previous_level

          hash[name] = level
          level
        end

        hash
      end

      def default
        get default_name
      end

      def each &block
        index.each_value &block
      end

      def get name
        index[name]
      end
      alias_method :[], :get

      def index
        @index ||= build_index
      end

      def resolve reference
        case reference
        when Symbol then get reference
        when String then get reference.to_sym
        else get reference.name
        end
      end
    end
  end
end
