class ExtendedLogger
  class Registry < Module
    attr_reader :attr_name
    attr_reader :logger_class
    attr_reader :sink

    def initialize logger_class, sink, attr_name
      @attr_name = attr_name
      @logger_class = logger_class
      @sink = sink
    end

    def self.build logger_class, attr_name: nil, **sink_arguments
      attr_name ||= :logger

      sink = logger_class.sink **sink_arguments

      instance = new logger_class, sink, attr_name
      instance.send :const_set, :Logger, logger_class
      instance
    end

    def build_logger key
      logger = logger_class.new sink
      logger.label = key.name
      logger
    end

    def cache
      @cache ||= Hash.new do |cache, key|
        cache[key] = build_logger key
      end
    end

    def clear
      cache.clear
    end

    def included cls
      cls.class_exec self do |registry|
        ivar_name = "@#{registry.attr_name}"
        cvar_name = "@@#{registry.attr_name}"

        attr_writer registry.attr_name

        define_method registry.attr_name do
          if instance_variable_defined? ivar_name
            return instance_variable_get ivar_name
          end

          if self.is_a? Module
            target = self
          else
            target = self.class
          end

          if target.class_variable_defined? cvar_name
            return target.class_variable_get cvar_name
          end

          logger = registry.get self
          self.class.class_variable_set cvar_name, logger
        end
      end
    end

    def extended object
      included object.singleton_class
    end

    def get receiver=nil
      receiver ||= TOPLEVEL_BINDING.receiver

      key = self.key receiver

      cache[key]
    end

    def key receiver
      if receiver.is_a? Module
        receiver
      else
        receiver.class
      end
    end

    def level
      sink.level
    end

    def level= reference
      sink.level = levels.resolve reference
    end

    def levels
      sink.levels
    end

    def lower_verbosity
      sink.lower_verbosity
    end

    def raise_verbosity
      sink.raise_verbosity
    end

    def set receiver, value
      key = self.key receiver

      cache[key] = value
    end
  end
end
