class ExtendedLogger
  class Serializer
    attr_writer :color_scheme
    attr_reader :log_format

    def initialize log_format
      @log_format = log_format
    end

    def self.build log_format: nil, color_scheme: nil
      log_format ||= Defaults.log_format

      instance = new log_format
      instance.color_scheme = color_scheme if color_scheme
      instance
    end

    def call message
      messages = format_text message

      messages.reduce String.new do |string, message|
        color_scheme.colorize message

        string << log_format.(message)
        string << "\n"
        string
      end
    end

    def color_scheme
      @color_scheme ||= ColorScheme::None
    end

    def format_text message
      lines = TextFormatter.(message.prose)

      lines.map do |line|
        line_message = message.dup
        line_message.prose = line
        line_message
      end
    end
  end
end
