class ExtendedLogger
  class Sink
    attr_accessor :device
    attr_reader :levels
    attr_writer :rules
    attr_reader :serializer

    def initialize device, levels, serializer
      @device = device
      @levels = levels
      @serializer = serializer
    end

    def self.build device=nil, color_scheme: nil, format: nil, levels: nil, rules: nil
      device ||= $stderr
      levels ||= Defaults.levels

      device = File.open device, 'a+' if device.is_a? String
      device.sync = true

      if color_scheme == :none or not (device.tty? or device.is_a? StringIO)
        color_scheme = nil
      end

      serializer = Serializer.build log_format: format, color_scheme: color_scheme

      instance = new device, levels, serializer
      instance.rules = rules if rules
      instance
    end

    def << message
      message = apply_rules message

      verbosity_threshold.(message) or return

      output = serializer.(message)

      device.write output
    end

    def apply_rules message
      rules.reduce message do |message, rule|
        rule.(message) or return
      end
    end

    def level
      verbosity_threshold.level
    end

    def level= reference
      verbosity_threshold.level = levels.resolve reference
    end

    def lower_verbosity
      verbosity_threshold.lower
    end

    def raise_verbosity
      verbosity_threshold.raise
    end

    def rules
      @rules ||= []
    end

    def verbosity_threshold
      @verbosity_threshold ||= VerbosityThreshold.new levels.default
    end

    Assertions = ExtendedLogger::Assertions
  end
end
